﻿using System;

using OpenPop.Mime;
using OpenPop.Mime.Traverse;

namespace TestProject
{
    /// <summary>
    /// 메시지 횡단자
    /// </summary>
    public class MessageTraverser : IAnswerMessageTraverser<MessageNode>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 메시지 파트 방문하기 - VisitMessagePart(messagePart)

        /// <summary>
        /// 메시지 파트 방문하기
        /// </summary>
        /// <param name="messagePart">메시지 파트</param>
        /// <returns>메시지 노드</returns>
        public MessageNode VisitMessagePart(MessagePart messagePart)
        {
            if(messagePart == null)
            {
                throw new ArgumentNullException("messagePart");
            }

            MessageNode node = new MessageNode();

            node.NodeName    = messagePart.ContentType.MediaType;
            node.MessagePart = messagePart;

            if(messagePart.IsMultiPart)
            {
                for(int i = 0; i< messagePart.MessageParts.Count; i++)
                {
                    MessageNode childNode = VisitMessagePart(messagePart.MessageParts[i]);

                    node.ChildNodeList.Add(childNode);
                }
            }

            return node;
        }

        #endregion
        #region 메시지 방문하기 - VisitMessage(message)

        /// <summary>
        /// 메시지 방문하기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <returns>메시지 노드</returns>
        public MessageNode VisitMessage(Message message)
        {
            if(message == null)
            {
                throw new ArgumentNullException("message");
            }

            MessageNode childNode = VisitMessagePart(message.MessagePart);

            MessageNode node = new MessageNode();
            
            node.NodeName    = message.Headers.Subject;
            node.MessagePart = message.MessagePart;

            node.ChildNodeList.Add(childNode);
            
            return node;
        }

        #endregion
    }
}